function [f,X,fs,fmx,fmn] = Wheel_FS2(data_tr,data_ts,NoW,NP,Strategy,classif,LoadFile,SaveFile,FNoF,indices)
% Feature subset selection using differential evolution and a wheel based
% search strategy
% Outputs:
% f - optimal fitness
% X - optimal solution
% fs - size of the best subset in each iteration
% fmx - size of the biggest subset of the population in each iteration
% fmn - size of the smallest subset of the population in each iteration
%
% Inputs:
% data_tr - training samples (No. of training x No. of features +1). (Last
% columns is the class label).
% data_ts - testing samples (No. of training x No. of features +1).
% NoW - No. of wheels (maximum number of features)
% NP - Population size
% Strategy - 'DE': Differential Evolution. 'GA': Genetic Algorithm.
% classif - 'SVM', 'KNN', 'NB'
% LoadFile - Name of file to load initial population from
% SaveFile - Name of file to save population and performance achieved in
% each iteration
% FNoF - Fixed No. of features. True: constrained wheels. False: non-constrained.
% indices - If indices of the K-fold cross-validation need to be provided
%
% Our implementation grows from the source code written by Ahmed Al-Ani.
% Reference: Ahmed Al-Ani, Akram Alsukker, Rami N. Khushaba. "Feature subset selection
% using differential evolution and a wheel based search strategy". Swarm and Evolutionary Computation


GEN = 600; % number of generations=iterations
[Ninst, NF] = size(data_tr);
NF = NF - 1;
Rot = max(4,round(GEN/120));
GEN1 = round(GEN/Rot);
% [CC,ps] = mapminmax(data_tr(:,1:NF)',0,1);data_tr(:,1:NF) = CC';
% CC = mapminmax('apply',data_ts(:,1:NF)',ps);data_ts(:,1:NF) = CC';
if nargin < 3 || isempty(NoW),
    NoW = floor(NF/2);
end
if nargin < 4 || isempty(NP),
    NP = 50;
end

if FNoF && NoW > NF/2
    error('For constrained variable selection, NoW should be <= NF/2');
end
if nargin < 10 || isempty(indices),
    CVF = 10;       % No. of cross-validation folds
    %TP = size(data_tr,1);
    %Tp1 = floor(TP/CVF);
    indices = crossvalind('Kfold',data_tr(:,end),CVF);
else
    if isempty(data_ts) && isempty(LoadFile),
        CVF = max(indices);
    else
        CVF = 0;
        indices = ones(Ninst,1);
    end
end
ff = zeros(1,GEN);
fs = zeros(1,GEN);
fmx = zeros(1,GEN);
fmn = zeros(1,GEN);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Check if there are features that have constant values for all patterns -s
in = false(NF,1);
for k=1:NF,
    dd = unique(data_tr(:,k));
    if length(dd) == 1,
        in(k) = true;
    end
end
if sum(in),
    find(in)
    warning('The above features have constant values in the training set');
end
% Check if there are features that have constant values for all patterns -e
% *************************** %
% ** ALGORITHM???S VARIABLES ** %
% *************************** %
X = zeros(NoW,1); % trial vector
Pop = zeros(NoW,NP); % population
Pin = Pop;
Pfin = Pop;
Fit = zeros(1,NP); % fitness of the population
%r = zeros(3,1); % randomly selected indices
% *********************** %
% ** CREATE POPULATION ** %
% *********************** %
nfw = floor(NF/NoW);%NF;%
tt = rem(NF,NoW);%0;%
NFw(1:NoW) = nfw;
NFw(1:tt) = nfw + 1;
ORD = randperm(NF);
W1 = reshape(ORD(1:NoW*nfw),NoW,nfw);
W1 = W1';
if ~FNoF,
    W1(nfw+1,tt+1:end) = NF*3;
else
    W1(nfw+1,tt+1:end) = NF*10;
end
if tt,
   W1(nfw+1,1:tt) = ORD(NoW*nfw+1:NF);
   if ~FNoF,
       W1(nfw+2,1:tt) = NF*3;
       W1(nfw+2,tt+1:end) = NF*10;
   end
end
if ~FNoF
    NFw = NFw + 1;
end
dw = ceil(NFw/2);
NE = round(NP*0.1); %round(NP*.125); number of elite members to keep while changing the wheels
% initialize random number generator
xx = boolean(0);
rand('state',sum(100*clock));
Ssz = zeros(1,NP);
for q = 1:Rot
   if ~isempty(LoadFile) && ~xx,
      str = sprintf('load %s Pop W1 NFw Pfin Pin indices CVF',LoadFile);
      eval(str);
      W2 = W1;    % copy of the wheel
      NFw2 = NFw;
   else
      if xx,
         Ssz1 = Ssz;
         for i=1:NE,
            Pop1(:,i) = Pop(:,bb(i));
            Pin1(:,i) = Pin(:,bb(i));
            Pfin1(:,i) = Pfin(:,bb(i));
            Ssz(i) = Ssz1(bb(i));
            Fit1(i) = Fit(bb(i));
         end
         % Generate new wheels and population
         Fit(1:NE) = Fit1;
         
         W1 = zeros(NFw(1),NoW) + NF*10;
         Pop = zeros(NoW,NP);
         Pin = Pop;
         Pfin = Pop;
         Temp = [];
         for k=1:NoW,
            Pop(k,1:NE) = Pop1(k,1:NE);
            temp = unique(Pop(k,1:NE));
            lt(k) = length(temp);
            W1(1:lt(k),k) = temp;
            Pin(k,1:NE) = grp2idx(Pop1(k,:));
            Pfin(k,1:NE) = Pfin1(k,:) - Pin1(k,:) + Pin(k,1:NE);
            Temp = [Temp temp];
         end
         Temp2 = setdiff(1:NF,Temp);
         ORD = randperm(length(Temp2));
         st = 1;
         for k=1:NoW,
             if ~FNoF,
                 t3 = sum(W1(1:lt(k),k) == NF*3);
                 ind = st:st-1+(NFw(k)-1) -(lt(k) - t3);
                 W1(lt(k)+1:NFw(k)-1+t3,k) = Temp2(ORD(ind));
                 W1(NFw(k):NFw(k)-t3,k) = NF*3;
                 st = st+(NFw(k)-1)-(lt(k)-t3);
             else
                 ind = st:st-1+NFw(k) -lt(k);
                 W1(lt(k)+1:NFw(k),k) = Temp2(ORD(ind));
                 st = st+NFw(k)-lt(k);
             end
         end
      end
      for j=1+xx*NE:NP,
         zq2 = randperm(NoW);%1:NoW;
         if ~FNoF,
             zq1 = randperm(NoW-5)+5;
             
             [F2, F3] = find(W1(:,zq2(1:NoW-zq1(1))) == NF*3);
             Pop(zq2(1:NoW-zq1(1)),j) = NF*3;
             Pin(zq2(1:NoW-zq1(1)),j) = F2;
             Pfin(zq2(1:NoW-zq1(1)),j) = F2;
         else
             zq1 = NoW;
         end
         for k=NoW-zq1(1)+1:NoW,
            k1 = zq2(k);
            rep = 1;
            while rep,
               F1 = 1 + (NFw(k1)-1)*rand; %1 + (NFw(k)-1)*rand;
               F2 = round(F1);
               if W1(F2,k1) < NF*3, % ~=
                  rep = 0;
               end
            end
            Pop(k1,j) = W1(F2,k1); % within wheel constraints
            Pin(k1,j) = F2; % index within each wheel
            Pfin(k1,j) = F1;
         end
      end
   end
   for j = 1+xx*NE:NP % initialize each individual
      val =round(Pop(:,j))';
      val = val(val<NF*3);
      Ssz(j) = length(val);
      fac = 0.00500*Ssz(j)/500;
      Fit(1,j) = Fit_classify(data_tr(:,val),data_ts(:,val),data_tr(:,end),data_ts(:,end),classif) + fac;

   end
   if ~xx,
      [aa, bb] = sort(Fit);
      iBest= bb(1);
      Best = Pop(:,iBest);
      %Bin = Pin(:,iBest);
      Bfin = Pfin(:,bb(1:NE));
      BSsz = Ssz(bb(1:NE));
      xx = boolean(1);
      if ~isempty(SaveFile),
          str = sprintf('save %s Pop W1 NFw Pfin Pin indices CVF',SaveFile);
          eval(str);
      end
      %save TabKW Pop W1 NFw Pfin Pin indices CVF
      fac = 0.00500*BSsz(1)/500;
      fprintf('%3d~ BAc: %.2f, AvAc: %.2f, mxSz:%d, mnSz:%d, BSz: %d\n',0,(1-Fit(iBest)+fac)*100,mean(1-Fit+0.00500*Ssz/500)*100,max(Ssz),min(Ssz),BSsz(1));
   end
   
   % ****************** %
   % ** OPTIMIZATION ** %
   % ****************** %
   for g = 1:GEN1 % for each generation
      expectation = ftskalr(Fit,NP);
      parents = ssuni(expectation,NP);
      parents = parents(randperm(length(parents)));
      for j = 1:NP % for each individual
         for vv = 1:10, %To avoid similarity with previous subsets. Try ten times before giving up
             switch Strategy,
                 case 'DE'
                     [VV, X] = GenerateNewPopulation_DE(NP,Ssz(j),j,Pfin,Bfin,NFw,dw,W1,NE,NF,parents);
                 case 'GA'
                     [VV, X] = GenerateNewPopulation_GA(j,Pin,NFw,W1,NE,bb,NF,parents, Pfin);
             end
             if max(sum(Pin == repmat(X,1,NP)))<NoW,
                 break
             end
         end
         val = find(VV);
         
         fac = 0.00500*length(val)/500;
         if ~CVF,
             f = Fit_classify(data_tr(:,val), data_ts(:,val), data_tr(:,end), data_ts(:,end), classif) + fac;
         else
             f = Fit_classifyCV(data_tr(:,val), data_tr(:,end), indices, classif) + fac;
         end
         % if trial is better or equal than current
         Pop2 = zeros(NoW,1);
         for i = 1:NoW
             Pop2(i) = W1(round(X(i)),i);
         end
         if f <= Fit(j) && max(sum(repmat(Pop2,1,NP)==Pop,1))<NoW,
            Pfin(:,j) = X;
            Pin(:,j) = round(X);
            Ssz(j) = length(val);
            Fit(j) = f;
            Pop(:,j) = Pop2;
         end
      end
      [aa,bb] = sort(Fit);
      iBest = bb(1);
      Best = Pop(:,iBest);
      Bfin = Pfin(:,bb(1:NE));
      BSsz = Ssz(bb(1:NE));
      fac = 0.00500*BSsz(1)/500;
      ff(g+(q-1)*GEN1) = (1-Fit(iBest)+fac)*100;
      fs(g+(q-1)*GEN1) =  BSsz(1);
      fmx(g+(q-1)*GEN1) =  max(Ssz);
      fmn(g+(q-1)*GEN1) =  min(Ssz);
      X = Best(Best<NF*3);
      fprintf('%3d~ BAc: %.2f, AvAc: %.2f, mxSz:%d, mnSz:%d, BSz: %d\n',g+(q-1)*GEN1,ff(g+(q-1)*GEN1),mean(1-Fit+0.00500*Ssz/500)*100,max(Ssz),min(Ssz),BSsz(1));
   end
   fprintf('\n');
   if ~isempty(SaveFile),
       str = sprintf('save %s Pop W1 NFw Pfin Pin indices CVF',SaveFile);
       eval(str);
   end
end
% ************* %
% ** RESULTS ** %
% ************* %
f = ff;
X = sort(Best(Best<NF*3));
% ============================================== %
% ******************* FUNCTIONS ******************* %
%-----------------------------------------------------------%
% %%%%%%%%%%%% GenerateNewPopulation_DE %%%%%%%%%%%%
%-----------------------------------------------------------%
function [VV, X] = GenerateNewPopulation_DE(NP,Ssz,j,Pfin,Bfin,NFw,dw,W1,NE,NF,parents)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%w_start = 0.95;                          %Initial inertia weight's value
%w_end = 0.35;                            %Final inertia weight
%w_varyfor = 1;
%w_varyfor = floor(w_varyfor*GEN);       %Weight change step. Defines total number of iterations for which weight is changed.
%w_now = w_start;
%inertdec = (.95-.35)/w_varyfor;           %Inertia weight's change per iteration
%w_start = 0.35;                          %Initial inertia weight's value
%w_end = 0.95;    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
CR = 0.5; % crossover constant
p1 = 0.1952; p2 = -13.32; p3 = 439.7; q1 = -17.82; q2 = 449.7;
rr = (p1*Ssz^2 + p2*Ssz + p3) / (Ssz^2 + q1*Ssz + q2);
clear p1 p2 q1 q2 p3
D = length(NFw);
% choose three random individuals from population,
% mutually different and different from j
%if (g<=w_varyfor) && (g > 1)
%   w_now = w_now + inertdec; %Change inertia weight
%end
r = floor(rand()* NP) + 1;
while (r==j)
   r = floor(rand()* NP) + 1;
end
% Roulette Wheel %
% create trial individual in which at least one parameter is changed
Rnd = floor(rand()*D) + 1;
rd = randperm(NE);
X = zeros(D,1);
val = zeros(D,1);
for i = 1:D
   if ( rand()>CR ) || ( Rnd==i )
      dd = Bfin(i,rd(1)) - Pfin(i,r);
      if dd > dw(i),
         dd = NFw(i) - dd;
      elseif dd < -dw(i),
         dd = -NFw(i) - dd;
      end
      X(i) = Pfin(i,j) + (rr*rand) * dd ;
      %X(i) = Pfin(i,parents(j)) + (rr*rand) * dd ;
      xs1 = X(i) > NFw(i)+0.5;
      xs2 = X(i) < 0.5;
      X(i) = (1-xs1)*X(i) + xs1*(X(i) - NFw(i));
      X(i) = (1-xs2)*X(i) + xs2*(X(i) + NFw(i));
   else
      if rem(j,4),
          rv = floor(rand()* NP) + 1;
          X(i) = Pfin(i,parents(rv));
      else
          X(i) = Pfin(i,parents(j));
      end
      %X(i) = Pfin(i,j);
   end
end
% Evaluate the generated subset
for i = 1:D
   val(i) = W1(round(X(i)),i);
end
val = val(val<NF*3);
VV = false(1,NF);
VV(val) = true;
%-----------------------------------------------------------%
% %%%%%%%%%%%% GenerateNewPopulation_GA %%%%%%%%%%%%
%-----------------------------------------------------------%
function [VV, child] = GenerateNewPopulation_GA(j,Pin,NFw,W1,NE,bb,NF,parents,Pfin)
Pc = 0.9;%0.8;
Pm = 0.05;
D = size(Pin,1);
NP = size(Pin,2);
RealCode = false;%true;%
CR_Type = 2;%1;%2;
child = zeros(D,1);
if rand()>Pc, %perform mutation
   if RealCode, % if real coded
      child = Pfin(:,j);
      rndm = rand(1,D);
      pp = find(rndm < Pm);
      child(pp) = rand(1,length(pp))*3+0.5;
   else      % else, binary coded
      child = Pin(:,j);
      rndm = rand(1,D);
      pp = find(rndm < Pm);
      for p=1:length(pp),
         in = randperm(NFw(pp(p)));
         in = in(in~=child(pp(p)));
         child(pp(p)) = in(1);%W1(in(1),pp(p));
      end
   end
else % perform crossover
   if CR_Type == 1;
       % option1: Randomly select two parents
       parents = parents(randperm(length(parents)));
       p1 = parents(1);
       p2 = parents(2);
       if p1 == p2,
           for m=3:length(parents),
               p2 = parents(m);
               if p1 ~= p2,
                   break,
               end
           end
       end
       if RealCode, % if real coded
           prt1 = Pfin(:,p1);
           prt2 = Pfin(:,p2);
       else         % else binary coded
           prt1 = Pin(:,p1);
           prt2 = Pin(:,p2);
       end
   else
       % option2: The first parent is j
       rd = randperm(NE);      %op2
       while bb(rd(1)) == j,      %op2
           rd = randperm(NE);      %op2
           %prt2 = Pin(:,bb(rd(1)));1      %op2
       end      %op2
       
       prt1 = Pin(:,j);      %op2
       prt2 = Pin(:,bb(rd(1)));      %op2
   end
   
   if RealCode,
       temp = rand(D,1);
       child = Pfin(:,j).*temp + Pfin(:,bb(rd(1))).*(1-temp);
   else
       in = rand(1,D)>0.25;%0.5;
       child(in) = prt1(in);
       %child(~in) = prt2(~in);
       for m=1:D,
           if ~in(m),
               rv = floor(rand()* NE) + 1;
               %prt2 = Pin(m,parents(rv)); 
               prt2 = Pin(m,bb(rv)); 
               child(m) = prt2;
           end
       end
   end
end
val = zeros(D,1);
for i = 1:D
   %fprintf('%d', i, child(i));
   val(i) = W1(round(child(i)),i);
end
val = val(val<NF*3);
VV = false(1,NF);
VV(val) = true;

%-----------------------------------------------------------%
% %%%%%%%%%%%% Fit_classify %%%%%%%%%%%%
%-----------------------------------------------------------%
function Fit = Fit_classify(data_tr, data_ts, data_trLab, data_tsLab, classif)
switch classif
   case 'KNN'
      %Class =knnclassify(Sample, Training, Group, k, distance,rule)
      Ac1 = knnclassify(data_ts,data_tr, data_trLab, 3);
      Fit = sum(Ac1~=data_tsLab)/size(data_ts,1);
   case 'SVM'
      model = train(data_trLab,sparse(data_tr));
      [TestPredict,a] = predict(data_tsLab,sparse(data_ts),model);
      Fit = (100-a)/100;
   case 'NB'    
        Obj1 = NaiveBayes.fit(data_tr,data_trLab);%,'dist',F);
        Ac1 = Obj1.predict(data_ts);
        Fit = sum(Ac1~=data_tsLab)/size(data_ts,1);
end

Fit = mean(Fit);
%-----------------------------------------------------------%
function expectation = ftskalr(scores,nParents)
[unused,i] = sort(scores);
expectation = zeros(size(scores));
expectation(i) = 1 ./ ((1:length(scores))  .^ 0.5);
expectation = nParents * expectation ./ sum(expectation);

function parents = ssuni(expectation,nParents)
wheel = cumsum(expectation) / nParents;
parents = zeros(1,nParents);
% we will step through the wheel in even steps.
stepSize = 1/nParents;
% we will start at a random position less that one full step
position = rand * stepSize;
% a speed optimization. Position is monotonically rising.
lowest = 1; 
for i = 1:nParents % for each parent needed,
    for j = lowest:length(wheel) % find the wheel position
        if(position < wheel(j)) % that this step falls in.
            parents(i) = j;
            lowest = j;
            break;
        end
    end
    position = position + stepSize; % take the next step.
end
